
// EchoLPServerMIDlet.java
// Andrew Davison, ad@fivedots.coe.psu.ac.th, September 2005 

/* The MIDlet starts the echo server.
   It displays the current number of handlers, and a
   scrollable text box of messages that it has received.

   Pressing the "exit" command causes the server and its handlers
   to terminate.

   The main difference between EchoLPServerMIDlet and EchoLServerMIDlet
   is only apparent in the server code in EchoLPServer. The server 
   only uses a single handler to process the messages coming from
   all the connected clients.

   --------------------
   Based on EchoLServerMIDlet
     - changed the class name

   Changed the variable names:
     numClientsSI --> numClientsSI 
     numClients --> numClients

   since there's only one handler on the server side for all the 
   clients.


*/

import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;


public final class EchoLPServerMIDlet extends MIDlet 
                                  implements CommandListener 
{
  private Display display;
  private Form form;
  private StringItem numClientsSI, messageSI;
  private ScrollableMessagesBox scroller;
  private Command exitCmd;

  private EchoLPServer echoServer;  // the top-level echo server
  private int numClients = 0;   // number of clients connected to server


  public EchoLPServerMIDlet() 
  {
    // build GUI
    form = new Form("Bluetooth Echo L2CAP Server");
    int width = form.getWidth();
    int height = form.getHeight();

    numClientsSI = new StringItem("No. Clients: ", "0");
    form.append(numClientsSI);
    int strHeight = numClientsSI.getPreferredHeight();    // height of string item

    scroller = new ScrollableMessagesBox("Received Messages:", 
                                          width, height-strHeight-20);
                            // 20 is for the title and a bit of space
    form.append(scroller);

    exitCmd = new Command("Exit", Command.EXIT, 1);
    form.addCommand(exitCmd);
    form.setCommandListener(this);
  } // end of EchoLPServerMIDlet()


  protected void startApp() 
  {  
    display = Display.getDisplay(this);
    display.setCurrent(form);
    echoServer = new EchoLPServer(this);   // create the echo server
    echoServer.start();   // start it listening for connections
  } // end of startApp()


  protected void destroyApp(boolean unconditional) 
  { notifyDestroyed(); }

  protected void pauseApp() {}


  public void commandAction(Command c, Displayable d) 
  { if (c == exitCmd) {
      echoServer.closeDown();  // close down the server and its handler
      destroyApp(true);
    }
  }


  // -------------- methods called by the handler -----------------

  synchronized public void incrCount()
  // called when a new client connects
  { numClients++;
    numClientsSI.setText("" + numClients);
  }

  synchronized public void decrCount()
  // called when a client connection is about to close
  { numClients--;
    numClientsSI.setText("" + numClients);
  }

  synchronized public void showMessage(String msg)
  // used to show the message received by the handler
  { scroller.addMessage(msg); }

} // end of EchoLPServerMIDlet class
